// PictureLoaderView.cpp : implementation of the CPictureLoaderView class
//

#include "stdafx.h"
#include "PictureLoader.h"

#include "PictureLoaderDoc.h"
#include "PictureLoaderView.h"

#include "picture.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPictureLoaderView

IMPLEMENT_DYNCREATE(CPictureLoaderView, CView)

BEGIN_MESSAGE_MAP(CPictureLoaderView, CView)
	//{{AFX_MSG_MAP(CPictureLoaderView)
	ON_WM_SIZE()
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPictureLoaderView construction/destruction

CPictureLoaderView::CPictureLoaderView()
{
	m_cx = 0;
	m_cy = 0;
	m_pPicture = NULL;
}

CPictureLoaderView::~CPictureLoaderView()
{
	delete m_pPicture;
}

BOOL CPictureLoaderView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

CRect CPictureLoaderView::SizeRectWithConstantAspectRatio( CRect* rcScreen,
                                       CSize sizePicture,
                                       BOOL bCenter)
{
  CRect rect(rcScreen);
  double dWidth = rcScreen->Width();
  double dHeight = rcScreen->Height();
  double dAspectRatio = dWidth/dHeight;

  double dPictureWidth = sizePicture.cx;
  double dPictureHeight = sizePicture.cy;
  double dPictureAspectRatio = dPictureWidth/dPictureHeight;

  //If the aspect ratios are the same then the screen rectangle
  // will do, otherwise we need to calculate the new rectangle

  if (dPictureAspectRatio > dAspectRatio)
  {
    int nNewHeight = (int)(dWidth/dPictureWidth*dPictureHeight);
	int nCenteringFactor = 0;
	
	if (bCenter)
		nCenteringFactor = (rcScreen->Height() - nNewHeight) / 2;
    
	rect.SetRect( 0,
                  nCenteringFactor,
                  (int)dWidth,
                  nNewHeight + nCenteringFactor);

  }
  else if (dPictureAspectRatio < dAspectRatio)
  {
    int nNewWidth =  (int)(dHeight/dPictureHeight*dPictureWidth);
	int nCenteringFactor = 0;
	
	if (bCenter)
		nCenteringFactor = (rcScreen->Width() - nNewWidth) / 2;
    
	rect.SetRect( nCenteringFactor, 
                  0,
                  nNewWidth + nCenteringFactor,
                  (int)(dHeight));
  }

  return rect;
}



/////////////////////////////////////////////////////////////////////////////
// CPictureLoaderView drawing

void CPictureLoaderView::OnDraw(CDC* pDC)
{
	CPictureLoaderDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (m_pPicture != NULL)
	{
		// Get Picture Dimentions In Pixels
		m_pPicture->UpdateSizeOnDC(pDC);

		//Get the dimensions of the screen
		CRect rcScreen( 0, 0, m_cx, m_cy);

		//Get dimensions of Image
		CSize sizePicture(m_pPicture->m_Width, m_pPicture->m_Height); 

		//Create a black background
		CBrush backBrush;
		backBrush.CreateSolidBrush(RGB(0,0,0));
		pDC->FillRect(&rcScreen, &backBrush);

		CRect rcNewPictureRect = 
			SizeRectWithConstantAspectRatio(&rcScreen,
											sizePicture,
											TRUE);
		m_pPicture->Show(pDC, rcNewPictureRect);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CPictureLoaderView printing

BOOL CPictureLoaderView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CPictureLoaderView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CPictureLoaderView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CPictureLoaderView diagnostics

#ifdef _DEBUG
void CPictureLoaderView::AssertValid() const
{
	CView::AssertValid();
}

void CPictureLoaderView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CPictureLoaderDoc* CPictureLoaderView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CPictureLoaderDoc)));
	return (CPictureLoaderDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CPictureLoaderView message handlers

void CPictureLoaderView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	m_cx = cx;
	m_cy = cy;
}

void CPictureLoaderView::OnFileOpen() 
{
	CFileDialog FileDialog(TRUE);

	if (FileDialog.DoModal() == IDOK)
	{
		delete m_pPicture;
		m_pPicture = new CPicture;
		m_strFileName = FileDialog.GetPathName();
		m_pPicture->Load(m_strFileName);
		Invalidate();
	}	
}
